package eksempel;

import java.applet.Applet;
import java.awt.*;

import javax.swing.*;  

import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import org.apache.commons.codec.binary.Base64;
import org.openoces.opensign.certificate.plugin.pkcs12.Pkcs12CertificateHandler;
import org.openoces.opensign.certificate.support.suncrypto.SunCryptoSupport;
import org.openoces.opensign.client.applet.CallBackHandler;
import org.openoces.opensign.client.applet.OS;
import org.openoces.opensign.client.applet.dialogs.DialogFactory;
import org.openoces.opensign.client.applet.dialogs.InputValidator;
import org.openoces.opensign.crypto.DefaultSignatureAlgorithm;
import org.openoces.opensign.exceptions.UserCancel;
import org.openoces.opensign.xml.xmldsig.BasicSignatureGenerator;
import org.openoces.opensign.xml.xmldsig.SignatureGenerator;

/*
 * Denne klasse opretter et vindue til at tage imod input til indlæsning af XBRL dokumenter og
 * benytter derefter {@link StoreXBRLInvoker} til at foretage kaldet til webservicen.
 * 
 * Denne klasse anvender BasigSignatureGenerator fra {@link openoces.org} til generering af XML Digital Signatur.
 * Denne generator skal anvendes da den genererede signatur valideres af en matchende klasse i webservicen. 
 * 
 * @author Mikael Jørgensen  <mfj@nineconsult.dk>
 */
public class RegisterXBRLFront {
	   /**
     * Create the GUI and show it.  For thread safety,
     * this method should be invoked from the
     * event-dispatching thread.
     */
	static JFrame frame = null;
    private static void createAndShowGUI() {
    	
    	// Hoved ramme med titel
    	frame = new JFrame("XBRL eksempel klient");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Panel til input felter/boxe
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        Insets insets = new Insets(3, 8, 3, 8);
        c.insets = insets;
        
        // ActionListener til browse knap til at hente fil-sti
        class BrowseAction implements ActionListener {
        	private JFrame parent;
        	private JTextField field;
        	private File defaultDirctory;

        	public BrowseAction(JFrame parent, JTextField fileName){
        		super();
        		this.parent = parent;
        		this.field = fileName;
        		String defaultDirectoryName = System.getProperty("user.home") + "/testfiler/";
        		this.defaultDirctory = new File(defaultDirectoryName);
        	}

			public void actionPerformed(ActionEvent e) {
				// vis fil-vælger
		        JFileChooser jFileChooser = new JFileChooser();
		        if (this.defaultDirctory != null && this.defaultDirctory.exists()) {
                    jFileChooser.setCurrentDirectory(defaultDirctory);
                } else {
                    jFileChooser.setCurrentDirectory(null);
                }
		        jFileChooser.showOpenDialog(parent);
		        if (jFileChooser.getSelectedFile() != null) {
					field.setText(jFileChooser.getSelectedFile()
							.getAbsolutePath());
				}
			}
        }
        int gridy = 0;

        // Overskrift
        JLabel headLabel = new JLabel("Dialog til eksempel på implementering af Erhvervsstyrelsens system-system løsning");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = gridy;
        panel.add(headLabel, c);

        JLabel separator0 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator0, c);

        // XBRL dokument vælger
        JLabel xbrlLabel = new JLabel("XBRL dokument");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(xbrlLabel, c);
        final JTextField xbrlFile = new JTextField(35);
        c.weightx = 1;
        c.gridwidth=5;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(xbrlFile, c);
        JButton xbrlBrowse = new JButton("...");
        xbrlBrowse.addActionListener(new BrowseAction(frame, xbrlFile));
        c.weightx = 0.5;
        c.gridwidth = 1;
        c.gridx = 6;
        c.gridy = gridy;
        panel.add(xbrlBrowse, c);

        // Pdf vælger
        JLabel pdfLabel = new JLabel("PDF dokument (til ERST)");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(pdfLabel, c);
        final JTextField pdfFile = new JTextField(35);
        pdfFile.setText(System.getProperty("user.home") + "/testfiler/testpdf.pdf");
        c.weightx = 1;
        c.gridwidth=5;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(pdfFile, c);
        JButton pdfBrowse = new JButton("...");
        pdfBrowse.addActionListener(new BrowseAction(frame, pdfFile));
        c.weightx = 0.5;
        c.gridwidth = 1;
        c.gridx = 6;
        c.gridy = gridy;
        panel.add(pdfBrowse, c);
        
        // Sender certifikat
        JLabel ocesSendLabel = new JLabel("Afsender PKCS#12 certifikat fil");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(ocesSendLabel, c);
        final JTextField ocesSendFile = new JTextField(35);
        ocesSendFile.setText(System.getProperty("user.home") + "/testfiler/MOCES_gyldig.p12");
        c.weightx = 1;
        c.gridwidth=5;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(ocesSendFile, c);
        JButton ocesSendBrowse = new JButton("...");
        ocesSendBrowse.addActionListener(new BrowseAction(frame, ocesSendFile));
        c.weightx = 0.5;
        c.gridwidth = 1;
        c.gridx = 6;
        c.gridy = gridy;
        panel.add(ocesSendBrowse, c);
        
        // Sender certifikat password
        JLabel ocesSendPwLabel = new JLabel("Afsender certifikat password");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(ocesSendPwLabel, c);
        final JPasswordField ocesSendPw = new JPasswordField(15);
        ocesSendPw.setText("Test1234");
        c.weightx = 1;
        c.gridwidth=2;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(ocesSendPw, c);

        // Token
        JLabel tokenLabel = new JLabel("Token");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(tokenLabel, c);
        final JTextField token = new JTextField(35);
        c.weightx = 1;
        c.gridwidth=2;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(token, c);

        // Destination vælger
        JLabel separator1 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator1, c);

        JLabel destLabel = new JLabel("Data destination");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(destLabel, c);
        final JRadioButton eogsBox = new JRadioButton("ERST", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(eogsBox, c);
        final JRadioButton dstBox = new JRadioButton("DST", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        panel.add(dstBox, c);
        final JRadioButton skatBox = new JRadioButton("SKAT", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        panel.add(skatBox, c);

		ButtonGroup destgroup = new ButtonGroup();
		destgroup.add(eogsBox);
		destgroup.add(dstBox);
		destgroup.add(skatBox);

        // service vælger
        JLabel separator2 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(separator2, c);

        JLabel serviceLabel = new JLabel("Service der rammes");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;

        panel.add(serviceLabel, c);

        final JRadioButton localRadio = new JRadioButton("localhost (til udviklingsbrug)", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(localRadio, c);
        final JRadioButton devRadio = new JRadioButton("archdev (ERST Dev)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(devRadio, c);
        final JRadioButton testRadio = new JRadioButton("archtest (ERST Test)", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(testRadio, c);
        final JRadioButton preprodRadio = new JRadioButton("archpreprod (ERST Preprod)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(preprodRadio, c);
        final JRadioButton prodRadio = new JRadioButton("archprod (ERST Produktion!)", true);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy++;
        panel.add(prodRadio, c);

        ButtonGroup group = new ButtonGroup();
        group.add(localRadio);
        group.add(devRadio);

        group.add(testRadio);
        group.add(preprodRadio);
        group.add(prodRadio);

        // Angiv om transport via https
        JLabel separator3 = new JLabel(" ");
        c.weightx = 1;
        c.gridwidth = 7;
        c.gridx = 0;
        c.gridy = gridy;
        panel.add(separator3, c);

        JLabel transportLabel = new JLabel("Transport");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 0;
        c.gridy = ++gridy;
        panel.add(transportLabel, c);
        final JCheckBox httpsBox = new JCheckBox("HTTPS", false);
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = gridy;
        panel.add(httpsBox, c);
        
        // Indlæs knap
        JButton indlaes = new JButton("Indlæs");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        c.fill = GridBagConstraints.NONE;
        panel.add(indlaes, c);
        
        // Ryd knap
        JButton ryd = new JButton("Ryd indtastning");
        c.weightx = 1;
        c.gridwidth = 1;
        c.gridx = 1;
        c.gridy = ++gridy;
        c.fill = GridBagConstraints.NONE;
        panel.add(ryd, c);

        // ActionListener for indlæs knap - check (minimalt) input og initier pakning, signering, og upload af data
        class IndlaesAction implements ActionListener {
            private JFrame parent;

            public IndlaesAction(JFrame parent) {
                super();
                this.parent = parent;
            }

            public void actionPerformed(ActionEvent e) {
                // Valider input
                boolean allOk = true;
                // The XBRL, PDF for EogsS and OCES must have been loaded
                allOk = xbrlFile.getText() != null && !xbrlFile.getText().equals("")
                        && (pdfFile.getText() != null && !pdfFile.getText().equals("") || !eogsBox.isSelected())
                        && ocesSendFile.getText() != null && !ocesSendFile.getText().equals("")
                        && ocesSendPw.getText() != null && !ocesSendPw.getText().equals("");


                if (!allOk) {
                    String errorTxt = "Der SKAL vælges\n\tXBRL dokument\n\tAfsender certifikat og password" +
                            "\n\tsamt PDF dokument hvis destination er ERST";
                    JOptionPane.showMessageDialog(parent, errorTxt, "Der mangler obligatoriske felter", JOptionPane.ERROR_MESSAGE);
                } else {
                    // Håndter data
                    // Simpel fejl håndtering
                    String errorText = "";
                    // Hent XBRL indhold
                    String xbrl = "";
                    try {
                        xbrl = readTextFile(xbrlFile.getText());
                    } catch (IOException e1) {
                        errorText = "Kan ikke læse XBRL dokument: \n" + e1.getMessage();
                    }
                    // fjern muligt Byte Order Mark - burde evt checke og afgøre kodning ud fra BOM hvis tilstede
                    int i = xbrl.indexOf('<');
                    if (i > 0) {
                        xbrl = xbrl.substring(i);
                    }
                    String pdf = null;
                    // Hent pdf indhold og Base64 indkod det hvis til ERST
                    if (eogsBox.isSelected()) {
                        try {
                            File file = new File(pdfFile.getText());
                            FileInputStream fos = new FileInputStream(file);
                            byte[] pdfBytes = new byte[(int) file.length()];
                            int n = fos.read(pdfBytes);
                            if (n != pdfBytes.length)
                                throw new IOException(n + " bytes read but expected " + pdfBytes.length);
                            pdf = Base64.encodeBase64String(pdfBytes);
                        } catch (IOException e1) {
                            errorText = "Kan ikke læse pdf dokument: \n" + e1.getMessage();
                        }
                    }

                    // Opret XML Digitale signature på baggrund af sender og godkender certifikater
                    String xmlDSigSender = null;

                    try {
                        OS.setCryptoSupport((new SunCryptoSupport()));

                        // Hent indhold der skal signeres
                        StringBuffer contentToBeSigned = new StringBuffer(xbrl);

                        // Instantier signatur generator
                        SignatureGenerator generator = new BasicSignatureGenerator();

                        // Sender signatur
                        // Hent sender certifikat password
                        String senderKeyPassword = ocesSendPw.getText();

                        // Opret ny CallBackHandler, der har en justeret promptPassword metode - den normale i oces pakken forventer input i Applet'en
                        CallBackHandler senderHandler = new CallBackHandler() {
                            private String password = null;

                            public void userMessage(String header, String message, boolean error) {
                            }

                            public String promptPassword(String message, InputValidator validator) throws UserCancel {
                                if (password == null) {
                                    password = message;
                                }
                                return password;
                            }

                            public URL getDocumentBase() {
                                return null;
                            }

                            public Frame getFrame() {
                                return null;
                            }

                            public Applet getApplet() {
                                return null;
                            }

                            public void confirmedUserMessage(String header, String message) {
                            }

                            public void clearUserMessage(boolean error) {
                            }

                            public void alert(String message) {
                            }

                            public String getParameter(String key) {
                                return null;
                            }

                            public DialogFactory getDialogFactory() {
                                return null;
                            }

                            public void repaint() {
                            }
                        };

                        InputValidator inputValidator = new InputValidator() {

                            public String getErrorMsg() {
                                return "Test error msg";
                            }

                            public String getInfo() {
                                return "Test info";
                            }

                            public boolean isValid(String arg0) {
                                return true;
                            }

                        };

                        // Initialiser handler med password
                        senderHandler.promptPassword(senderKeyPassword, inputValidator);

                        // Hent sender Certificate
                        Pkcs12CertificateHandler senderCertHandler = new Pkcs12CertificateHandler(new File(ocesSendFile.getText()), senderKeyPassword.toCharArray());

                        // Sæt callback handler på certificatehandler
                        senderCertHandler.setCallBackHandler(senderHandler);

                        // Lav signatur
                        StringBuffer senderSignature = generator.sign(senderCertHandler, senderKeyPassword.toCharArray(), contentToBeSigned, null, new Properties(), null, new DefaultSignatureAlgorithm());
                        // Fjern xml prolog fra signaturen
                        xmlDSigSender = senderSignature.substring(40);
                        System.out.println(senderSignature.substring(0, 40));

                        FileOutputStream fos = new FileOutputStream(xbrlFile.getText() + "sign.xml");
                        fos.write(xmlDSigSender.getBytes("UTF-8"));
                        fos.close();

                    } catch (Exception e1) {
                        errorText = "Kan ikke generere digitale signaturer: \n" + e1.getMessage();
                    }

                    // Saml input
                    RegisterXBRLInput input = new RegisterXBRLInput();
                    input.setSendDst(dstBox.isSelected() ? "Y" : "N");
                    input.setSendEogS(eogsBox.isSelected() ? "Y" : "N");
                    input.setSendSkat(skatBox.isSelected() ? "Y" : "N");
                    input.setPdf(pdf);
                    input.setSignedXbrl(xmlDSigSender);
                    input.setToken(token.getText());

                    // Test for fejl
                    if (!errorText.equals("")) {
                        JOptionPane.showMessageDialog(parent, errorText, "Fejl ved behandling af data:", JOptionPane.ERROR_MESSAGE);
                    } else {
                        // Kald webservice
                        RegisterXBRLInvoker invoker = new RegisterXBRLInvoker();
                        String errorMsg = null;
                        String result = null;
                        try {
                            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                            result = invoker.registerXBRL(input, httpsBox.isSelected(), prodRadio.isSelected(), preprodRadio.isSelected(),
                                    testRadio.isSelected(), devRadio.isSelected(), localRadio.isSelected());
                            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        } catch (Exception e1) {
                            errorMsg = e1.getMessage();
                            frame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                        }

                        // Vis resultatet i dialog
                        if (errorMsg != null) {
                            JTextArea text = new JTextArea(errorMsg, 0, 100);
                            text.setLineWrap(true);
                            text.setSize(new Dimension(480, 10));
                            text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                            JOptionPane.showMessageDialog(parent, text, "Fejl ved kald af webservice", JOptionPane.ERROR_MESSAGE);
                        } else {
                            JTextArea text = new JTextArea(result, 0, 75);
                            text.setLineWrap(true);
                            text.setSize(new Dimension(480, 10));
                            text.setPreferredSize(new Dimension(480, text.getPreferredSize().height));
                            JOptionPane.showMessageDialog(parent, text, "Resultat af kald til webservice", JOptionPane.INFORMATION_MESSAGE);
                        }
                    }
                }

            }
        }

        indlaes.addActionListener(new IndlaesAction(frame));

        // ActionListener til ryd knap
        class RydAction implements ActionListener {
        	public RydAction(){
        		super();
        	}

			public void actionPerformed(ActionEvent e) {
				// Nulstil input 
				xbrlFile.setText("");
				pdfFile.setText("");
				ocesSendFile.setText("");
				ocesSendPw.setText("");
				eogsBox.setSelected(true);
				dstBox.setSelected(false);
				skatBox.setSelected(false);
				testRadio.setSelected(true);
            }
        }

        ryd.addActionListener(new RydAction());

        // Vis vinduet.
        frame.getContentPane().add(panel);
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
    
    // Hjælper metode til at indlæse en fil
    private static String readTextFile(String fullPathFilename) throws IOException {
    	StringBuffer sb = new StringBuffer(1024);
    	BufferedReader reader = new BufferedReader(new InputStreamReader( new FileInputStream(fullPathFilename),"UTF-8"));
    			
    	String line = null;
    	while( ( line = reader.readLine()) != null){
    		if (sb.length() != 0) {
				sb.append(System.getProperty("line.separator"));
			}
			sb.append(line);
    	}
    	reader.close();
    	return sb.toString();
    }
}
