package eksempel;

import javax.xml.bind.DatatypeConverter;
import javax.xml.rpc.ServiceException;

import webservice_xbrl.registerXbrlEkstern.RegisterXbrlEksternWS;
import webservice_xbrl.registerXbrlEkstern.RegisterXbrlEksternWSServiceLocator;
import webservice_xbrl.registerXbrlEkstern.WsReturnRegister;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.io.File;
import java.io.FileOutputStream;

//import org.apache.commons.codec.binary.Base64;



/*
 * Denne klasse anvendes til at kalde webservices via de klasser der er genereret udfra den publiserede wsdl.
 *
 * @author Mikael Jørgensen  <mfj@nineconsult.dk>
 */
public class RegisterXBRLInvoker {

	public RegisterXBRLInvoker() {
		super();

	}
	/*
	 * Metode der, ved brug af klasser genereret på basis af den publiserede wsdl, modtager argumenter og kalder webservicen.
	 *
	 * @param approvedOCESSignatur XML Digital Signatur der signerer XBRL dokumentet med godkenders certifikat
	 * @param senderOCESSignatur XML Digital Signatur der signerer XBRL dokumentet med senders certifikat
	 * @param storeXBRLInputs En {link StoreXBRLInput} der indeholder XBRL dokumentet samt tilhørende data
	 * @param useHttps Angivelse af om kaldet til webservicen skal gå via https eller http
	 * @param clientCertPath Sti til placering af klient certifikat til brug for ssl/https
	 * @param clientCertPw Password til klient certifikat til brug for ssl/https
	 * @return retur værdi fra webservicen
	 */
	public String registerXBRL(RegisterXBRLInput registerXbrlInput, boolean useHttps, boolean useProd, boolean usePreprod, boolean useTest, boolean useDev, boolean useLocal) throws Exception{

		// Resultat af webservice kaldet
		String result = "";

		try {
			// Dette er endpoints for serverne, i de forskellige miljøer
			String prodEndpoint    = "://erst.virk.dk/ri/s2s/RegisterXbrlEksternWS";
			String preprodEndpoint = "://erstpreprod.virk.dk/ri/s2s/RegisterXbrlEksternWS";
			String testEndpoint    = "://ersttest.virk.dk/ri/s2s/RegisterXbrlEksternWS";
			String devEndpoint     = "://erstdev.virk.dk/ri/s2s/RegisterXbrlEksternWS";
			String localEndpoint   = "://localhost:9029/regnskab-indberetning-web/s2s/RegisterXbrlEksternWS";

			String endpointToUse;
			if (useProd){
				endpointToUse = prodEndpoint;
			} else if (usePreprod){
				endpointToUse = preprodEndpoint;
			} else if (useTest){
				endpointToUse = testEndpoint;
			} else if (useDev){
				endpointToUse = devEndpoint;
			} else {
				endpointToUse = localEndpoint;
			}
			System.out.println("Benytter endpoint: " + endpointToUse);

			// Hent stub til kald af webservicen
			RegisterXbrlEksternWSServiceLocator stubLocator = new RegisterXbrlEksternWSServiceLocator();
			if (useHttps) {
				// Sæt endpoint - i det publiserede wsdl er endpointet den interne esb adresse
				stubLocator.setEndpointAddress("RegisterXbrlEksternWSPort", "https"+endpointToUse);
			} else {
				// Sæt endpoint - i det publiserede wsdl er endpointet den interne adresse
				stubLocator.setEndpointAddress("RegisterXbrlEksternWSPort", "http"+endpointToUse);
			}
			RegisterXbrlEksternWS stub = stubLocator.getRegisterXbrlEksternWSPort();

			// Kald webservicen
			WsReturnRegister returnStatus;
			String invocationTime = "WS kaldet tog ";
			try {
				long t1 = System.currentTimeMillis();
				returnStatus = stub.registerXbrlEkstern(registerXbrlInput.getSignedXbrl(), registerXbrlInput.getPdf(),
						registerXbrlInput.getSendEogS(), registerXbrlInput.getSendDst(), registerXbrlInput.getSendSkat(), registerXbrlInput.getToken());
				long duration = System.currentTimeMillis()-t1;
				invocationTime = invocationTime + (duration/1000)+" sekunder\n";

			} catch (Exception e) {
				e.printStackTrace();
				throw new Exception(e.getMessage());
			}

			// Opsaml retur værdier
			String skatKvitMsg = "";
			String skatKvittering = returnStatus.getSkatKvittering();
			if (skatKvittering != null) {
				skatKvitMsg = "\nKvittering fra SKAT er modtaget og lagt i filen " + gemSkatKvittering(skatKvittering);
			}
			String skatKvit = returnStatus.getSkatKvittering()!=null ? "\nKvittering fra SKAT er modtaget" : "";
			result = invocationTime+
					"Code: "+returnStatus.getStatusCode()+
					"\nText: "+returnStatus.getStatusText()+
					"\nDetail:\n"+returnStatus.getStatusDetail() + skatKvitMsg +
					"\nToken: " + returnStatus.getToken() +
					"\nSagsnummer: " + returnStatus.getUniqueReportIdentifier();
			Clipboard clpbrd = Toolkit.getDefaultToolkit().getSystemClipboard ();
			StringSelection stringSelection = new StringSelection(returnStatus.getUniqueReportIdentifier());
			clpbrd.setContents(stringSelection, null);

		} catch (ServiceException e) {
			e.printStackTrace();
		}

		return result;
	}

	/*
	 * Gem base64 encoded PDF kvittering fra SKAT i temporær fil.
	 */
	String gemSkatKvittering(String kvit) throws java.io.IOException {
		File file = File.createTempFile("skatKvittering_",".pdf");
		byte[] bytes = DatatypeConverter.parseBase64Binary(kvit);
		FileOutputStream fos = new FileOutputStream(file);
		fos.write(bytes);
		fos.close();
		return file.getAbsolutePath();
	}

}